@@ -10,10 +10,14 @@ this script configure the environment variables for GLRM
 
 
 
+import os
+import sys
+
+
+
 """
 overwrite the print function with the default set to flush = True
 """
-import sys
 def print(*objects, sep=' ', end='\n', file=sys.stdout, flush=True):
     import builtins
     builtins.print(*objects, sep=sep, end=end, file=file, flush=flush)
@@ -22,13 +26,14 @@ def print(*objects, sep=' ', end='\n', file=sys.stdout, flush=True):
 
 """
 version info
+UPDATE: read it from file version.py rather than directly define it here
 """
-cur_version = '1.0.1'
+from version import __version__
+cur_version = __version__
 
 
 
 # The [is-docker package for npm](https://github.com/sindresorhus/is-docker/blob/master/index.js) suggests a robust approach to determine if it's running within a docker container
-import os
 def is_docker():
     path = '/proc/self/cgroup'
     return (
@@ -50,6 +55,8 @@ else:
     input_path = ''
     output_path = os.getcwd()
 
+# diagnosis plots folder
+diagnosis_path = os.path.join(output_path, 'diagnosis')
 
 
 """
@@ -84,8 +91,8 @@ gamma = 4e-3
 """
 define number of digits used for round floats in fast computing the heavy-tail probabilities
 """
-mu_digits = 10
-sigma2_digits = 6
+mu_digits = 3
+sigma2_digits = 2
 
 
 
@@ -99,6 +106,7 @@ sigma2_eps = 1e-8
 
 """
 configuration for reproducible results in keras + TensorFlow
+skipped here, we configure the environment in main function
 """
 # Seed value
 #seed_value = 1154
