@@ -5,19 +5,32 @@ Created on Tue May 10 03:15:36 2022
 
 @author: hill103
 
-this script is the main function of the whole GLRM pipeline
+this script is the main function of the whole CVAE-GLRM pipeline
 
 pipeline steps:
     
-    1. receive and parse the command line parameters and then pass the parameters to GLRM model
-    2. pre-process (include building CVAE)
-    3. model fitting
-    4. post-process and save the results
+    1. receive and parse the command line parameters and then pass the parameters to CVAE-GLRM model
+    2. pre-process
+    3. building CVAE (if available)
+    4. GLRM model fitting
+    5. post-process and save the results
 """
 
 
 
 def main():
+    '''
+    main function
+    
+    Parameters
+    ----------
+    None.
+
+    Returns
+    -------
+    None.
+
+    '''
 
     from config import print, cur_version, output_path
     import os
@@ -52,7 +65,7 @@ def main():
     
     
     print('\n\n######### Preprocessing... #########\n')
-    data = preprocess(paramdict['spatial_file'], paramdict['ref_file'], paramdict['ref_celltype_file'], paramdict['marker_file'], paramdict['A_file'], paramdict['use_cvae'], paramdict['n_hv_gene'], paramdict['n_marker_per_cmp'], paramdict['pseudo_spot_min_cell'], paramdict['pseudo_spot_max_cell'], paramdict['seq_depth_scaler'], paramdict['cvae_input_scaler'], paramdict['cvae_init_lr'], paramdict['redo_de'], paramdict['diagnosis'])
+    data = preprocess(paramdict['spatial_file'], paramdict['ref_file'], paramdict['ref_celltype_file'], paramdict['marker_file'], paramdict['A_file'], paramdict['use_cvae'], paramdict['n_hv_gene'], paramdict['n_marker_per_cmp'], paramdict['n_pseudo_spot'], paramdict['pseudo_spot_min_cell'], paramdict['pseudo_spot_max_cell'], paramdict['seq_depth_scaler'], paramdict['cvae_input_scaler'], paramdict['cvae_init_lr'], paramdict['num_hidden_layer'], paramdict['use_batch_norm'], paramdict['cvae_train_epoch'], paramdict['use_spatial_pseudo'], paramdict['redo_de'], paramdict['use_fdr'], paramdict['p_val_cutoff'], paramdict['fc_cutoff'], paramdict['pct1_cutoff'], paramdict['pct2_cutoff'], paramdict['sortby_fc'], paramdict['filter_cell'], paramdict['filter_gene'], paramdict['diagnosis'])
     
     
     # whether to estimate gamma_g, if CVAE is used then disable gamma_g estimation
@@ -61,7 +74,7 @@ def main():
     else:
         estimate_gamma_g = True
     
-    
+
     # release RAM before modeling
     #import gc
     #import psutil
@@ -96,15 +109,17 @@ def main():
     # imputation
     if paramdict['use_imputation']:
         
-        from imputation import generate_imputation_expression_map
+        from imputation import do_imputation
         
         print('\n\n######### Start imputation #########')
         
         for x in paramdict['impute_diameter']:
+            print(f'\n\nimputation for {x} µm ...')
             impute_start = time()
-            # 1 µm = 10 pixels
-            result = generate_imputation_expression_map(paramdict['loc_file'], output_file, paramdict['spatial_file'], paramdict['diameter']*10, x*10)
-            # return imputed spot locations, cell-type proportions and gene expressions
+            # we now totally discard the transforming from integer coordinates to pixels
+            # we use stepsize = impute_diameter / diameter inside imputation function
+            result = do_imputation(paramdict['loc_file'], output_file, paramdict['spatial_file'], float(x)/paramdict['diameter'], paramdict['hole_min_spots'], paramdict['preserve_shape'], diagnosis=paramdict['diagnosis'])
+            # return imputed spot locations, cell type proportions and gene expressions
             result[0].to_csv(os.path.join(output_path, f'impute_diameter_{x}_spot_loc.csv'))
             result[1].to_csv(os.path.join(output_path, f'impute_diameter_{x}_spot_celltype_prop.csv'))
             result[2].to_csv(os.path.join(output_path, f'impute_diameter_{x}_spot_gene_norm_exp.csv'))
