@@ -15,7 +15,6 @@ the performance metric is RMSE of predicted gene expressions in validation fold
 
 
 
-import os
 import networkx as nx
 import numpy as np
 from scipy import sparse
@@ -24,7 +23,7 @@ from time import time
 from admm_fit import one_admm_fit
 from local_fit_numba import update_theta, hv_wrapper
 from utils import reportRMSE, calcRMSE
-from config import print, output_path
+from config import print
 
 
 
@@ -93,7 +92,7 @@ def calcHVBaseModelLoss(theta, e_alpha, y, mu, gamma_g, sigma2, hv_x, hv_log_p,
     non_zero_mtx : None or 2-D numpy matrix (spots * genes)
         If it's None, then do not filter zeros during regression. If it's a bool 2-D numpy matrix (spots * genes) as False means genes whose nUMI=0 while True means genes whose nUMI>0 in corresponding spots. The bool indicators can be calculated based on either observerd raw nUMI counts in spatial data, or CVAE transformed nUMI counts.
     use_cache : bool, optional
-        if True, use the cached dict of calculated negative log-likelihood values.
+        if True, use the cached dict of calculated likelihood values.
         
     Returns
     -------
@@ -131,15 +130,17 @@ def cv_find_lambda_r(data, mle_theta, mle_e_alpha, gamma_g, sigma2, lasso_weight
     
     Parameters
     ----------
-    data : a Dict contains all info need for modeling
-        X: a 2-D numpy matrix of celltype specific marker gene expression (celltypes * genes).
-        Y: a 2-D numpy matrix of spatial gene expression (spots * genes).
-        A: a 2-D numpy matrix of Adjacency matrix (spots * spots), or is None. Adjacency matrix of spatial sptots (1: connected / 0: disconnected). All 0 in diagonal.
-        N: a 1-D numpy array of sequencing depth of all spots (length #spots). If it's None, use sum of observed marker gene expressions as sequencing depth.
-        non_zero_mtx: If it's None, then do not filter zeros during regression. If it's a bool 2-D numpy matrix (spots * genes) as False means genes whose nUMI=0 while True means genes whose nUMI>0 in corresponding spots. The bool indicators can be calculated based on either observerd raw nUMI counts in spatial data, or CVAE transformed nUMI counts.
-        spot_names: a list of string of spot barcodes. Only keep spots passed filtering.
-        gene_names: a list of string of gene symbols. Only keep actually used marker genes.
-        celltype_names: a list of string of celltype names.
+    data : Dict
+        a Dict contains all info need for modeling:
+            X: a 2-D numpy matrix of celltype specific marker gene expression (celltypes * genes).\n
+            Y: a 2-D numpy matrix of spatial gene expression (spots * genes).\n
+            A: a 2-D numpy matrix of Adjacency matrix (spots * spots), or is None. Adjacency matrix of spatial sptots (1: connected / 0: disconnected). All 0 in diagonal.\n
+            N: a 1-D numpy array of sequencing depth of all spots (length #spots). If it's None, use sum of observed marker gene expressions as sequencing depth.\n
+            non_zero_mtx: If it's None, then do not filter zeros during regression. If it's a bool 2-D numpy matrix (spots * genes) as False means genes whose nUMI=0 while True means genes whose nUMI>0 in corresponding spots. The bool indicators can be calculated based on either observerd raw nUMI counts in spatial data, or CVAE transformed nUMI counts.\n
+            spot_names: a list of string of spot barcodes. Only keep spots passed filtering.\n
+            gene_names: a list of string of gene symbols. Only keep actually used marker genes.\n
+            celltype_names: a list of string of celltype names.\n
+            initial_guess: initial guess of cell-type proportions of spatial spots.
     mle_theta : 3-D numpy array (spots * celltypes * 1)
         estimated theta (celltype proportion) by MLE.
     mle_e_alpha : 1-D numpy array
@@ -167,7 +168,7 @@ def cv_find_lambda_r(data, mle_theta, mle_e_alpha, gamma_g, sigma2, lasso_weight
     k : int, optional
         the number of folds in cross-validation, The default value is 5.
     use_cache : bool, optional
-        if True, use the cached dict of calculated negative log-likelihood values.
+        if True, use the cached dict of calculated likelihood values.
     diagnosis : bool
         if True save more information to files for diagnosis CVAE and hyper-parameter selection
         
@@ -308,47 +309,43 @@ def cv_find_lambda_r(data, mle_theta, mle_e_alpha, gamma_g, sigma2, lasso_weight
         
     # draw plot
     if diagnosis:
-        import matplotlib.pyplot as plt
-        import seaborn as sns
-        sns.set()
         
-        fig, ax = plt.subplots()
-        ax.plot(candidate_list[:len(avg_rmse_list)], avg_rmse_list, marker='o')
-        # log scale do not support 0, so use symlog
-        ax.set_xscale('symlog', linthresh=candidate_list[1])
-        ax.set_xticks(candidate_list)
-        ax.set_xticklabels(candidate_list, rotation=45)
-        ax.axvline(candidate_list[optimal_idx], color='red', linestyle='--')
-        
-        ax.set_xlabel('lambda_r')
         if use_likelihood:
-            ax.set_ylabel('average negative log-likelihood')
+            y_label = 'average negative log-likelihood'
         else:
-            ax.set_ylabel('average theta RMSE')
+            y_label = 'average theta RMSE'
+        
+        from diagnosis_plots import diagnosisParamsTuning
+        diagnosisParamsTuning(candidate_list, avg_rmse_list, optimal_idx, 'lambda_r', y_label)
         
-        fig.tight_layout()
-        fig.savefig(os.path.join(output_path, 'tuning lambda_r.png'))
-        plt.close()
 
     return candidate_list[optimal_idx]
     
     
     
-def cv_find_lambda_g(data, G, theta_mask, gamma_g, sigma2, candidate_list, hybrid_version=True, opt_method='L-BFGS-B', hv_x=None, hv_log_p=None, use_admm=True, use_likelihood=True, k=5, use_cache=True, diagnosis=False):
+def cv_find_lambda_g(data, G, stage1_theta, stage1_e_alpha, theta_mask, gamma_g, sigma2, candidate_list, hybrid_version=True, opt_method='L-BFGS-B', hv_x=None, hv_log_p=None, use_admm=True, use_likelihood=True, k=5, use_cache=True, diagnosis=False):
     '''
     find optimal value for hyper-parameter lambda_g by k fold cross-validation
     
     Parameters
     ----------
-    data : a Dict contains all info need for modeling
-        X: a 2-D numpy matrix of celltype specific marker gene expression (celltypes * genes).
-        Y: a 2-D numpy matrix of spatial gene expression (spots * genes).
-        A: a 2-D numpy matrix of Adjacency matrix (spots * spots), or is None. Adjacency matrix of spatial sptots (1: connected / 0: disconnected). All 0 in diagonal.
-        N: a 1-D numpy array of sequencing depth of all spots (length #spots). If it's None, use sum of observed marker gene expressions as sequencing depth.
-        non_zero_mtx: If it's None, then do not filter zeros during regression. If it's a bool 2-D numpy matrix (spots * genes) as False means genes whose nUMI=0 while True means genes whose nUMI>0 in corresponding spots. The bool indicators can be calculated based on either observerd raw nUMI counts in spatial data, or CVAE transformed nUMI counts.
-        spot_names: a list of string of spot barcodes. Only keep spots passed filtering.
-        gene_names: a list of string of gene symbols. Only keep actually used marker genes.
-        celltype_names: a list of string of celltype names.
+    data : Dict
+        a Dict contains all info need for modeling:
+            X: a 2-D numpy matrix of celltype specific marker gene expression (celltypes * genes).\n
+            Y: a 2-D numpy matrix of spatial gene expression (spots * genes).\n
+            A: a 2-D numpy matrix of Adjacency matrix (spots * spots), or is None. Adjacency matrix of spatial sptots (1: connected / 0: disconnected). All 0 in diagonal.\n
+            N: a 1-D numpy array of sequencing depth of all spots (length #spots). If it's None, use sum of observed marker gene expressions as sequencing depth.\n
+            non_zero_mtx: If it's None, then do not filter zeros during regression. If it's a bool 2-D numpy matrix (spots * genes) as False means genes whose nUMI=0 while True means genes whose nUMI>0 in corresponding spots. The bool indicators can be calculated based on either observerd raw nUMI counts in spatial data, or CVAE transformed nUMI counts.\n
+            spot_names: a list of string of spot barcodes. Only keep spots passed filtering.\n
+            gene_names: a list of string of gene symbols. Only keep actually used marker genes.\n
+            celltype_names: a list of string of celltype names.\n
+            initial_guess: initial guess of cell-type proportions of spatial spots.
+    G : built graph object from networks module
+        used for constructing Laplacian Matrix.
+    stage1_theta : 3-D numpy array (spots * celltypes * 1)
+        estimated theta (celltype proportion) in stage 1.
+    stage1_e_alpha : 1-D numpy array
+        estimated e_alpha (spot-specific effect) in stage 1.
     theta_mask : 3-D numpy array (spots * celltypes * 1)
         mask for cell-type proportions (1: present, 0: not present). Only used for stage 2 theta optmization.
     gamma_g : 1-D numpy array
@@ -372,7 +369,7 @@ def cv_find_lambda_g(data, G, theta_mask, gamma_g, sigma2, candidate_list, hybri
     k : int, optional
         the number of folds in cross-validation, The default value is 5.
     use_cache : bool, optional
-        if True, use the cached dict of calculated negative log-likelihood values.
+        if True, use the cached dict of calculated likelihood values.
     diagnosis : bool
         if True save more information to files for diagnosis CVAE and hyper-parameter selection
         
@@ -398,12 +395,18 @@ def cv_find_lambda_g(data, G, theta_mask, gamma_g, sigma2, candidate_list, hybri
     n_spot, n_gene = data['Y'].shape
     
     # re-initialize theta and e_alpha for only present cell-types
+    # update: reuse the result from stage 1
+    '''
     start_theta = np.zeros(theta_mask.shape)
     for i in range(n_spot):
         start_theta[i, theta_mask[i,:,:]==1] = 1.0/np.sum(theta_mask[i,:,:])
     
     start_e_alpha = np.full((n_spot,), 1.0)
-
+    '''
+    
+    start_theta = stage1_theta.copy()
+    start_e_alpha = stage1_e_alpha.copy()
+    
     # random permute the genes
     gene_idx = np.arange(n_gene)
     # set seed for reproducibility
@@ -533,26 +536,14 @@ def cv_find_lambda_g(data, G, theta_mask, gamma_g, sigma2, candidate_list, hybri
         
     # draw plot
     if diagnosis:
-        import matplotlib.pyplot as plt
-        import seaborn as sns
-        sns.set()
         
-        fig, ax = plt.subplots()
-        ax.plot(candidate_list[:len(avg_rmse_list)], avg_rmse_list, marker='o')
-        # log scale do not support 0, so use symlog
-        ax.set_xscale('symlog', linthresh=candidate_list[1])
-        ax.set_xticks(candidate_list)
-        ax.set_xticklabels(candidate_list, rotation=45)
-        ax.axvline(candidate_list[optimal_idx], color='red', linestyle='--')
-        
-        ax.set_xlabel('lambda_g')
         if use_likelihood:
-            ax.set_ylabel('average negative log-likelihood')
+            y_label = 'average negative log-likelihood'
         else:
-            ax.set_ylabel('average theta RMSE')
+            y_label = 'average theta RMSE'
         
-        fig.tight_layout()
-        fig.savefig(os.path.join(output_path, 'tuning lambda_g.png'))
-        plt.close()
+        from diagnosis_plots import diagnosisParamsTuning
+        diagnosisParamsTuning(candidate_list, avg_rmse_list, optimal_idx, 'lambda_g', y_label)
+    
     
     return candidate_list[optimal_idx]
\ No newline at end of file
